part of '../flutterx_reflection.dart';

/// Field of a class
///
/// [type] of this field
/// [name] of this field
/// [getter] getter implementation having type as return type and receiver as argument
/// [setter] setter implementation having as parameter the receiver and the new value.
///          If not set it means this field is final
/// [isFinal] if [setter] is null
/// [toString] describes this field accurately through [_describeFlutterField] function
/// [get] simply invoke the [getter] and return field value
/// [set] simply invoke the [setter] and set new field value
@immutable
class FlutterField {
  final FlutterType type;
  final String name;
  final Function getter;
  final Function? setter;

  bool get isFinal => setter == null;

  const FlutterField(this.type, this.name, this.getter, [this.setter]);

  @override
  String toString() => _describeFlutterField(this);

  T get<T>(dynamic receiver) => getter(receiver);

  void set(dynamic receiver, dynamic value) =>
      requireNotNull(setter, message: () => 'cannot set a final field')(receiver, value);
}
